@@ -8,8 +8,8 @@ import contextlib
 import inspect
 from functools import reduce
 import os
+import pathlib
 import sys
-import types
 
 import pytest
 
@@ -27,34 +27,25 @@ from .fixtures import django_db_modify_db_settings_xdist_suffix  # noqa
 from .fixtures import _live_server_helper  # noqa
 from .fixtures import admin_client  # noqa
 from .fixtures import admin_user  # noqa
+from .fixtures import async_client  # noqa
 from .fixtures import client  # noqa
 from .fixtures import db  # noqa
 from .fixtures import django_user_model  # noqa
 from .fixtures import django_username_field  # noqa
 from .fixtures import live_server  # noqa
 from .fixtures import django_db_reset_sequences  # noqa
+from .fixtures import async_rf  # noqa
 from .fixtures import rf  # noqa
 from .fixtures import settings  # noqa
 from .fixtures import transactional_db  # noqa
 
 from .lazy_django import django_settings_is_configured, skip_if_no_django
 
-try:
-    import pathlib
-except ImportError:
-    import pathlib2 as pathlib
-
 
 SETTINGS_MODULE_ENV = "DJANGO_SETTINGS_MODULE"
 CONFIGURATION_ENV = "DJANGO_CONFIGURATION"
 INVALID_TEMPLATE_VARS_ENV = "FAIL_INVALID_TEMPLATE_VARS"
 
-PY2 = sys.version_info[0] == 2
-
-# pytest 4.2 handles unittest setup/teardown itself via wrapping fixtures.
-_pytest_version_info = tuple(int(x) for x in pytest.__version__.split(".", 2)[:2])
-_handle_unittest_methods = _pytest_version_info < (4, 2)
-
 _report_header = []
 
 
@@ -128,6 +119,12 @@ def pytest_addoption(parser):
         type="bool",
         default=True,
     )
+    parser.addini(
+        "django_debug_mode",
+        "How to set the Django DEBUG setting (default `False`). "
+        "Use `keep` to not override.",
+        default="False",
+    )
     group.addoption(
         "--fail-on-template-vars",
         action="store_true",
@@ -240,7 +237,7 @@ def _get_boolean_value(x, name, default=None):
     except KeyError:
         raise ValueError(
             "{} is not a valid value for {}. "
-            "It must be one of {}." % (x, name, ", ".join(possible_values.keys()))
+            "It must be one of {}.".format(x, name, ", ".join(possible_values.keys()))
         )
 
 
@@ -303,11 +300,11 @@ def pytest_load_initial_conftests(early_config, parser, args):
     dc, dc_source = _get_option_with_source(options.dc, CONFIGURATION_ENV)
 
     if ds:
-        _report_header.append("settings: %s (from %s)" % (ds, ds_source))
+        _report_header.append("settings: {} (from {})".format(ds, ds_source))
         os.environ[SETTINGS_MODULE_ENV] = ds
 
         if dc:
-            _report_header.append("configuration: %s (from %s)" % (dc, dc_source))
+            _report_header.append("configuration: {} (from {})".format(dc, dc_source))
             os.environ[CONFIGURATION_ENV] = dc
 
             # Install the django-configurations importer
@@ -330,91 +327,13 @@ def pytest_report_header():
         return ["django: " + ", ".join(_report_header)]
 
 
-@pytest.mark.trylast
+@pytest.hookimpl(trylast=True)
 def pytest_configure():
     # Allow Django settings to be configured in a user pytest_configure call,
     # but make sure we call django.setup()
     _setup_django()
 
 
-def _classmethod_is_defined_at_leaf(cls, method_name):
-    super_method = None
-
-    for base_cls in cls.__mro__[1:]:  # pragma: no branch
-        super_method = base_cls.__dict__.get(method_name)
-        if super_method is not None:
-            break
-
-    assert super_method is not None, (
-        "%s could not be found in base classes" % method_name
-    )
-
-    method = getattr(cls, method_name)
-
-    try:
-        f = method.__func__
-    except AttributeError:
-        pytest.fail("%s.%s should be a classmethod" % (cls, method_name))
-    if PY2 and not (
-        inspect.ismethod(method)
-        and inspect.isclass(method.__self__)
-        and issubclass(cls, method.__self__)
-    ):
-        pytest.fail("%s.%s should be a classmethod" % (cls, method_name))
-    return f is not super_method.__func__
-
-
-_disabled_classmethods = {}
-
-
-def _disable_class_methods(cls):
-    if cls in _disabled_classmethods:
-        return
-
-    _disabled_classmethods[cls] = (
-        # Get the classmethod object (not the resulting bound method),
-        # otherwise inheritance will be broken when restoring.
-        cls.__dict__.get("setUpClass"),
-        _classmethod_is_defined_at_leaf(cls, "setUpClass"),
-        cls.__dict__.get("tearDownClass"),
-        _classmethod_is_defined_at_leaf(cls, "tearDownClass"),
-    )
-
-    cls.setUpClass = types.MethodType(lambda cls: None, cls)
-    cls.tearDownClass = types.MethodType(lambda cls: None, cls)
-
-
-def _restore_class_methods(cls):
-    (
-        setUpClass,
-        restore_setUpClass,
-        tearDownClass,
-        restore_tearDownClass,
-    ) = _disabled_classmethods.pop(cls)
-
-    try:
-        del cls.setUpClass
-    except AttributeError:
-        raise
-
-    try:
-        del cls.tearDownClass
-    except AttributeError:
-        pass
-
-    if restore_setUpClass:
-        cls.setUpClass = setUpClass
-
-    if restore_tearDownClass:
-        cls.tearDownClass = tearDownClass
-
-
-def pytest_runtest_setup(item):
-    if _handle_unittest_methods:
-        if django_settings_is_configured() and is_django_unittest(item):
-            _disable_class_methods(item.cls)
-
-
 @pytest.hookimpl(tryfirst=True)
 def pytest_collection_modifyitems(items):
     # If Django is not configured we don't need to bother
@@ -467,11 +386,15 @@ def django_test_environment(request):
     """
     if django_settings_is_configured():
         _setup_django()
-        from django.conf import settings as dj_settings
         from django.test.utils import setup_test_environment, teardown_test_environment
 
-        dj_settings.DEBUG = False
-        setup_test_environment()
+        debug_ini = request.config.getini("django_debug_mode")
+        if debug_ini == "keep":
+            debug = None
+        else:
+            debug = _get_boolean_value(debug_ini, False)
+
+        setup_test_environment(debug=debug)
         request.addfinalizer(teardown_test_environment)
 
 
@@ -523,33 +446,21 @@ def _django_setup_unittest(request, django_db_blocker):
     # Fix/patch pytest.
     # Before pytest 5.4: https://github.com/pytest-dev/pytest/issues/5991
     # After pytest 5.4: https://github.com/pytest-dev/pytest-django/issues/824
-    from _pytest.monkeypatch import MonkeyPatch
+    from _pytest.unittest import TestCaseFunction
+    original_runtest = TestCaseFunction.runtest
 
     def non_debugging_runtest(self):
         self._testcase(result=self)
 
-    mp_debug = MonkeyPatch()
-    mp_debug.setattr("_pytest.unittest.TestCaseFunction.runtest", non_debugging_runtest)
+    try:
+        TestCaseFunction.runtest = non_debugging_runtest
 
-    request.getfixturevalue("django_db_setup")
-
-    cls = request.node.cls
-
-    with django_db_blocker.unblock():
-        if _handle_unittest_methods:
-            _restore_class_methods(cls)
-            cls.setUpClass()
-            _disable_class_methods(cls)
+        request.getfixturevalue("django_db_setup")
 
+        with django_db_blocker.unblock():
             yield
-
-            _restore_class_methods(cls)
-            cls.tearDownClass()
-        else:
-            yield
-
-    if mp_debug:
-        mp_debug.undo()
+    finally:
+        TestCaseFunction.runtest = original_runtest
 
 
 @pytest.fixture(scope="function", autouse=True)
@@ -591,12 +502,7 @@ def _django_set_urlconf(request):
     if marker:
         skip_if_no_django()
         import django.conf
-
-        try:
-            from django.urls import clear_url_caches, set_urlconf
-        except ImportError:
-            # Removed in Django 2.0
-            from django.core.urlresolvers import clear_url_caches, set_urlconf
+        from django.urls import clear_url_caches, set_urlconf
 
         urls = validate_urls(marker)
         original_urlconf = django.conf.settings.ROOT_URLCONF
@@ -629,14 +535,13 @@ def _fail_for_invalid_template_variable():
     ``pytest.mark.ignore_template_errors``
     """
 
-    class InvalidVarException(object):
+    class InvalidVarException:
         """Custom handler for invalid strings in templates."""
 
         def __init__(self):
             self.fail = True
 
         def __contains__(self, key):
-            """There is a test for '%s' in TEMPLATE_STRING_IF_INVALID."""
             return key == "%s"
 
         @staticmethod
@@ -674,10 +579,9 @@ def _fail_for_invalid_template_variable():
                 return template.name
 
         def __mod__(self, var):
-            """Handle TEMPLATE_STRING_IF_INVALID % var."""
             origin = self._get_origin()
             if origin:
-                msg = "Undefined template variable '%s' in '%s'" % (var, origin)
+                msg = "Undefined template variable '{}' in '{}'".format(var, origin)
             else:
                 msg = "Undefined template variable '%s'" % var
             if self.fail:
@@ -692,11 +596,7 @@ def _fail_for_invalid_template_variable():
         from django.conf import settings as dj_settings
 
         if dj_settings.TEMPLATES:
-            dj_settings.TEMPLATES[0]["OPTIONS"][
-                "string_if_invalid"
-            ] = InvalidVarException()
-        else:
-            dj_settings.TEMPLATE_STRING_IF_INVALID = InvalidVarException()
+            dj_settings.TEMPLATES[0]["OPTIONS"]["string_if_invalid"] = InvalidVarException()
 
 
 @pytest.fixture(autouse=True)
@@ -710,8 +610,6 @@ def _template_string_if_invalid_marker(request):
 
             if dj_settings.TEMPLATES:
                 dj_settings.TEMPLATES[0]["OPTIONS"]["string_if_invalid"].fail = False
-            else:
-                dj_settings.TEMPLATE_STRING_IF_INVALID.fail = False
 
 
 @pytest.fixture(autouse=True, scope="function")
@@ -732,7 +630,7 @@ def _django_clear_site_cache():
 # ############### Helper Functions ################
 
 
-class _DatabaseBlockerContextManager(object):
+class _DatabaseBlockerContextManager:
     def __init__(self, db_blocker):
         self._db_blocker = db_blocker
 
@@ -743,7 +641,7 @@ class _DatabaseBlockerContextManager(object):
         self._db_blocker.restore()
 
 
-class _DatabaseBlocker(object):
+class _DatabaseBlocker:
     """Manager for django.db.backends.base.base.BaseDatabaseWrapper.
 
     This is the object returned by django_db_blocker.
